;(function(){

    /**
     * Require the given path.
     *
     * @param {String} path
     * @return {Object} exports
     * @api public
     */

    function require(path, parent, orig) {
        var resolved = require.resolve(path);

        // lookup failed
        if (null == resolved) {
            orig = orig || path;
            parent = parent || 'root';
            var err = new Error('Failed to require "' + orig + '" from "' + parent + '"');
            err.path = orig;
            err.parent = parent;
            err.require = true;
            throw err;
        }

        var module = require.modules[resolved];

        // perform real require()
        // by invoking the module's
        // registered function
        if (!module._resolving && !module.exports) {
            var mod = {};
            mod.exports = {};
            mod.client = mod.component = true;
            module._resolving = true;
            module.call(this, mod.exports, require.relative(resolved), mod);
            delete module._resolving;
            module.exports = mod.exports;
        }

        return module.exports;
    }

    /**
     * Registered modules.
     */

    require.modules = {};

    /**
     * Registered aliases.
     */

    require.aliases = {};

    /**
     * Resolve `path`.
     *
     * Lookup:
     *
     *   - PATH/index.js
     *   - PATH.js
     *   - PATH
     *
     * @param {String} path
     * @return {String} path or null
     * @api private
     */

    require.resolve = function(path) {
        if (path.charAt(0) === '/') path = path.slice(1);

        var paths = [
            path,
            path + '.js',
            path + '.json',
            path + '/index.js',
            path + '/index.json'
        ];

        for (var i = 0; i < paths.length; i++) {
            var path = paths[i];
            if (require.modules.hasOwnProperty(path)) return path;
            if (require.aliases.hasOwnProperty(path)) return require.aliases[path];
        }
    };

    /**
     * Normalize `path` relative to the current path.
     *
     * @param {String} curr
     * @param {String} path
     * @return {String}
     * @api private
     */

    require.normalize = function(curr, path) {
        var segs = [];

        if ('.' != path.charAt(0)) return path;

        curr = curr.split('/');
        path = path.split('/');

        for (var i = 0; i < path.length; ++i) {
            if ('..' == path[i]) {
                curr.pop();
            } else if ('.' != path[i] && '' != path[i]) {
                segs.push(path[i]);
            }
        }

        return curr.concat(segs).join('/');
    };

    /**
     * Register module at `path` with callback `definition`.
     *
     * @param {String} path
     * @param {Function} definition
     * @api private
     */

    require.register = function(path, definition) {
        require.modules[path] = definition;
    };

    /**
     * Alias a module definition.
     *
     * @param {String} from
     * @param {String} to
     * @api private
     */

    require.alias = function(from, to) {
        if (!require.modules.hasOwnProperty(from)) {
            throw new Error('Failed to alias "' + from + '", it does not exist');
        }
        require.aliases[to] = from;
    };

    /**
     * Return a require function relative to the `parent` path.
     *
     * @param {String} parent
     * @return {Function}
     * @api private
     */

    require.relative = function(parent) {
        var p = require.normalize(parent, '..');

        /**
         * lastIndexOf helper.
         */

        function lastIndexOf(arr, obj) {
            var i = arr.length;
            while (i--) {
                if (arr[i] === obj) return i;
            }
            return -1;
        }

        /**
         * The relative require() itself.
         */

        function localRequire(path) {
            var resolved = localRequire.resolve(path);
            return require(resolved, parent, path);
        }

        /**
         * Resolve relative to the parent.
         */

        localRequire.resolve = function(path) {
            var c = path.charAt(0);
            if ('/' == c) return path.slice(1);
            if ('.' == c) return require.normalize(p, path);

            // resolve deps by returning
            // the dep in the nearest "deps"
            // directory
            var segs = parent.split('/');
            var i = lastIndexOf(segs, 'deps') + 1;
            if (!i) i = 0;
            path = segs.slice(0, i + 1).join('/') + '/deps/' + path;
            return path;
        };

        /**
         * Check if module is defined at `path`.
         */

        localRequire.exists = function(path) {
            return require.modules.hasOwnProperty(localRequire.resolve(path));
        };

        return localRequire;
    };
    require.register("component-indexof/index.js", function(exports, require, module){
        module.exports = function(arr, obj){
            if (arr.indexOf) return arr.indexOf(obj);
            for (var i = 0; i < arr.length; ++i) {
                if (arr[i] === obj) return i;
            }
            return -1;
        };
    });
    require.register("component-classes/index.js", function(exports, require, module){
        /**
         * Module dependencies.
         */

        var index = require('indexof');

        /**
         * Whitespace regexp.
         */

        var re = /\s+/;

        /**
         * toString reference.
         */

        var toString = Object.prototype.toString;

        /**
         * Wrap `el` in a `ClassList`.
         *
         * @param {Element} el
         * @return {ClassList}
         * @api public
         */

        module.exports = function(el){
            return new ClassList(el);
        };

        /**
         * Initialize a new ClassList for `el`.
         *
         * @param {Element} el
         * @api private
         */

        function ClassList(el) {
            if (!el) throw new Error('A DOM element reference is required');
            this.el = el;
            this.list = el.classList;
        }

        /**
         * Add class `name` if not already present.
         *
         * @param {String} name
         * @return {ClassList}
         * @api public
         */

        ClassList.prototype.add = function(name){
            // classList
            if (this.list) {
                this.list.add(name);
                return this;
            }

            // fallback
            var arr = this.array();
            var i = index(arr, name);
            if (!~i) arr.push(name);
            this.el.className = arr.join(' ');
            return this;
        };

        /**
         * Remove class `name` when present, or
         * pass a regular expression to remove
         * any which match.
         *
         * @param {String|RegExp} name
         * @return {ClassList}
         * @api public
         */

        ClassList.prototype.remove = function(name){
            if ('[object RegExp]' == toString.call(name)) {
                return this.removeMatching(name);
            }

            // classList
            if (this.list) {
                this.list.remove(name);
                return this;
            }

            // fallback
            var arr = this.array();
            var i = index(arr, name);
            if (~i) arr.splice(i, 1);
            this.el.className = arr.join(' ');
            return this;
        };

        /**
         * Remove all classes matching `re`.
         *
         * @param {RegExp} re
         * @return {ClassList}
         * @api private
         */

        ClassList.prototype.removeMatching = function(re){
            var arr = this.array();
            for (var i = 0; i < arr.length; i++) {
                if (re.test(arr[i])) {
                    this.remove(arr[i]);
                }
            }
            return this;
        };

        /**
         * Toggle class `name`.
         *
         * @param {String} name
         * @return {ClassList}
         * @api public
         */

        ClassList.prototype.toggle = function(name){
            // classList
            if (this.list) {
                this.list.toggle(name);
                return this;
            }

            // fallback
            if (this.has(name)) {
                this.remove(name);
            } else {
                this.add(name);
            }
            return this;
        };

        /**
         * Return an array of classes.
         *
         * @return {Array}
         * @api public
         */

        ClassList.prototype.array = function(){
            var str = this.el.className.replace(/^\s+|\s+$/g, '');
            var arr = str.split(re);
            if ('' === arr[0]) arr.shift();
            return arr;
        };

        /**
         * Check if class `name` is present.
         *
         * @param {String} name
         * @return {ClassList}
         * @api public
         */

        ClassList.prototype.has =
            ClassList.prototype.contains = function(name){
                return this.list
                    ? this.list.contains(name)
                    : !! ~index(this.array(), name);
            };

    });
    require.register("segmentio-extend/index.js", function(exports, require, module){

        module.exports = function extend (object) {
            // Takes an unlimited number of extenders.
            var args = Array.prototype.slice.call(arguments, 1);

            // For each extender, copy their properties on our object.
            for (var i = 0, source; source = args[i]; i++) {
                if (!source) continue;
                for (var property in source) {
                    object[property] = source[property];
                }
            }

            return object;
        };
    });
    require.register("component-event/index.js", function(exports, require, module){
        var bind = (window.addEventListener !== undefined) ? 'addEventListener' : 'attachEvent',
            unbind = (window.removeEventListener !== undefined) ? 'removeEventListener' : 'detachEvent',
            prefix = (bind !== 'addEventListener') ? 'on' : '';

        /**
         * Bind `el` event `type` to `fn`.
         *
         * @param {Element} el
         * @param {String} type
         * @param {Function} fn
         * @param {Boolean} capture
         * @return {Function}
         * @api public
         */

        exports.bind = function(el, type, fn, capture){
            el[bind](prefix + type, fn, capture || false);

            return fn;
        };

        /**
         * Unbind `el` event `type`'s callback `fn`.
         *
         * @param {Element} el
         * @param {String} type
         * @param {Function} fn
         * @param {Boolean} capture
         * @return {Function}
         * @api public
         */

        exports.unbind = function(el, type, fn, capture){
            el[unbind](prefix + type, fn, capture || false);

            return fn;
        };
    });
    require.register("component-type/index.js", function(exports, require, module){

        /**
         * toString ref.
         */

        var toString = Object.prototype.toString;

        /**
         * Return the type of `val`.
         *
         * @param {Mixed} val
         * @return {String}
         * @api public
         */

        module.exports = function(val){
            switch (toString.call(val)) {
                case '[object Function]': return 'function';
                case '[object Date]': return 'date';
                case '[object RegExp]': return 'regexp';
                case '[object Arguments]': return 'arguments';
                case '[object Array]': return 'array';
                case '[object String]': return 'string';
            }

            if (val === null) return 'null';
            if (val === undefined) return 'undefined';
            if (val && val.nodeType === 1) return 'element';
            if (val === Object(val)) return 'object';

            return typeof val;
        };

    });
    require.register("timoxley-is-collection/index.js", function(exports, require, module){
        var typeOf = require('type')

        /**
         * Evaluates _obj_ to determine if it's an array, an array-like collection, or
         * something else. This is useful when working with the function `arguments`
         * collection and `HTMLElement` collections.
         * Note: This implementation doesn't consider elements that are also
         *
         *

         collections, such as `<form>` and `<select>`, to be array-like.

         @method test
         @param {Object} obj Object to test.
         @return {Number} A number indicating the results of the test:

         * 0: Neither an array nor an array-like collection.
         * 1: Real array.
         * 2: Array-like collection.

         @api private
         **/
        module.exports = function isCollection(obj) {
            var type = typeOf(obj)
            if (type === 'array') return 1
            switch (type) {
                case 'arguments': return 2
                case 'object':
                    if (isNodeList(obj)) return 2
                    try {
                        // indexed, but no tagName (element) or scrollTo/document (window. From DOM.isWindow test which we can't use here),
                        // or functions without apply/call (Safari
                        // HTMLElementCollection bug).
                        if ('length' in obj
                            && !obj.tagName
                            && !(obj.scrollTo && obj.document)
                            && !obj.apply) {
                            return 2
                        }
                    } catch (ex) {}
                default:
                    return 0
            }
        }

        function isNodeList(nodes) {
            return typeof nodes === 'object'
                && /^\[object (NodeList)\]$/.test(Object.prototype.toString.call(nodes))
                && nodes.hasOwnProperty('length')
                && (nodes.length == 0 || (typeof nodes[0] === "object" && nodes[0].nodeType > 0))
        }


    });
    require.register("javve-events/index.js", function(exports, require, module){
        var events = require('event'),
            isCollection = require('is-collection');

        /**
         * Bind `el` event `type` to `fn`.
         *
         * @param {Element} el, NodeList, HTMLCollection or Array
         * @param {String} type
         * @param {Function} fn
         * @param {Boolean} capture
         * @api public
         */

        exports.bind = function(el, type, fn, capture){
            if (!isCollection(el)) {
                events.bind(el, type, fn, capture);
            } else if ( el && el[0] !== undefined ) {
                for ( var i = 0; i < el.length; i++ ) {
                    events.bind(el[i], type, fn, capture);
                }
            }
        };

        /**
         * Unbind `el` event `type`'s callback `fn`.
         *
         * @param {Element} el, NodeList, HTMLCollection or Array
         * @param {String} type
         * @param {Function} fn
         * @param {Boolean} capture
         * @api public
         */

        exports.unbind = function(el, type, fn, capture){
            if (!isCollection(el)) {
                events.unbind(el, type, fn, capture);
            } else if ( el && el[0] !== undefined ) {
                for ( var i = 0; i < el.length; i++ ) {
                    events.unbind(el[i], type, fn, capture);
                }
            }
        };
    });
    require.register("javve-get-by-class/index.js", function(exports, require, module){
        /**
         * Find all elements with class `className` inside `container`.
         * Use `single = true` to increase performance in older browsers
         * when only one element is needed.
         *
         * @param {String} className
         * @param {Element} container
         * @param {Boolean} single
         * @api public
         */

        module.exports = (function() {
            if (document.getElementsByClassName) {
                return function(container, className, single) {
                    if (single) {
                        return container.getElementsByClassName(className)[0];
                    } else {
                        return container.getElementsByClassName(className);
                    }
                };
            } else if (document.querySelector) {
                return function(container, className, single) {
                    if (single) {
                        return container.querySelector(className);
                    } else {
                        return container.querySelectorAll(className);
                    }
                };
            } else {
                return function(container, className, single) {
                    var classElements = [],
                        tag = '*';
                    if (container == null) {
                        container = document;
                    }
                    var els = container.getElementsByTagName(tag);
                    var elsLen = els.length;
                    var pattern = new RegExp("(^|\\s)"+className+"(\\s|$)");
                    for (var i = 0, j = 0; i < elsLen; i++) {
                        if ( pattern.test(els[i].className) ) {
                            if (single) {
                                return els[i];
                            } else {
                                classElements[j] = els[i];
                                j++;
                            }
                        }
                    }
                    return classElements;
                };
            }
        })();

    });
    require.register("javve-to-string/index.js", function(exports, require, module){
        module.exports = function(s) {
            s = (s === undefined) ? "" : s;
            s = (s === null) ? "" : s;
            s = s.toString();
            return s;
        };

    });
    require.register("list.fuzzysearch.js/index.js", function(exports, require, module){
        var classes = require('classes'),
            events = require('events'),
            extend = require('extend'),
            toString = require('to-string'),
            getByClass = require('get-by-class');

        module.exports = function(options) {
            options = options || {};

            extend(options, {
                location: 0,
                distance: 100,
                threshold: 0.4,
                multiSearch: true,
                searchClass: 'fuzzy-search'
            });

            var fuzzy = require('./src/fuzzy'),
                list;

            var fuzzySearch = {
                search: function(searchString, columns) {
                    // Substract arguments from the searchString or put searchString as only argument
                    var searchArguments = options.multiSearch ? searchString.replace(/ +$/, '').split(/ +/) : [searchString];

                    for (var k = 0, kl = list.items.length; k < kl; k++) {
                        fuzzySearch.item(list.items[k], columns, searchArguments);
                    }
                },
                item: function(item, columns, searchArguments) {
                    var found = true;
                    for(var i = 0; i < searchArguments.length; i++) {
                        var foundArgument = false;
                        for (var j = 0, jl = columns.length; j < jl; j++) {
                            if (fuzzySearch.values(item.values(), columns[j], searchArguments[i])) {
                                foundArgument = true;
                            }
                        }
                        if(!foundArgument) {
                            found = false;
                        }
                    }
                    item.found = found;
                },
                values: function(values, value, searchArgument) {
                    if (values.hasOwnProperty(value)) {
                        var text = toString(values[value]).toLowerCase();

                        if (fuzzy(text, searchArgument, options)) {
                            return true;
                        }
                    }
                    return false;
                }
            };

            return {
                init: function(parentList) {
                    list = parentList;

                    events.bind(getByClass(list.listContainer, options.searchClass), 'keyup', function(e) {
                        var target = e.target || e.srcElement; // IE have srcElement
                        list.search(target.value, fuzzySearch.search);
                    });

                    return;
                },
                search: function(str, columns) {
                    list.search(str, columns, fuzzySearch.search);
                },
                name: options.name || "fuzzySearch"
            };
        };

    });
    require.register("list.fuzzysearch.js/src/fuzzy.js", function(exports, require, module){
        module.exports = function(text, pattern, options) {
            // Aproximately where in the text is the pattern expected to be found?
            var Match_Location = options.location || 0;

            //Determines how close the match must be to the fuzzy location (specified above). An exact letter match which is 'distance' characters away from the fuzzy location would score as a complete mismatch. A distance of '0' requires the match be at the exact location specified, a threshold of '1000' would require a perfect match to be within 800 characters of the fuzzy location to be found using a 0.8 threshold.
            var Match_Distance = options.distance || 100;

            // At what point does the match algorithm give up. A threshold of '0.0' requires a perfect match (of both letters and location), a threshold of '1.0' would match anything.
            var Match_Threshold = options.threshold || 0.4;

            if (pattern === text) return true; // Exact match
            if (pattern.length > 32) return false; // This algorithm cannot be used

            // Set starting location at beginning text and initialise the alphabet.
            var loc = Match_Location,
                s = (function() {
                    var q = {},
                        i;

                    for (i = 0; i < pattern.length; i++) {
                        q[pattern.charAt(i)] = 0;
                    }

                    for (i = 0; i < pattern.length; i++) {
                        q[pattern.charAt(i)] |= 1 << (pattern.length - i - 1);
                    }

                    return q;
                }());

            // Compute and return the score for a match with e errors and x location.
            // Accesses loc and pattern through being a closure.

            function match_bitapScore_(e, x) {
                var accuracy = e / pattern.length,
                    proximity = Math.abs(loc - x);

                if (!Match_Distance) {
                    // Dodge divide by zero error.
                    return proximity ? 1.0 : accuracy;
                }
                return accuracy + (proximity / Match_Distance);
            }

            var score_threshold = Match_Threshold, // Highest score beyond which we give up.
                best_loc = text.indexOf(pattern, loc); // Is there a nearby exact match? (speedup)

            if (best_loc != -1) {
                score_threshold = Math.min(match_bitapScore_(0, best_loc), score_threshold);
                // What about in the other direction? (speedup)
                best_loc = text.lastIndexOf(pattern, loc + pattern.length);

                if (best_loc != -1) {
                    score_threshold = Math.min(match_bitapScore_(0, best_loc), score_threshold);
                }
            }

            // Initialise the bit arrays.
            var matchmask = 1 << (pattern.length - 1);
            best_loc = -1;

            var bin_min, bin_mid;
            var bin_max = pattern.length + text.length;
            var last_rd;
            for (var d = 0; d < pattern.length; d++) {
                // Scan for the best match; each iteration allows for one more error.
                // Run a binary search to determine how far from 'loc' we can stray at this
                // error level.
                bin_min = 0;
                bin_mid = bin_max;
                while (bin_min < bin_mid) {
                    if (match_bitapScore_(d, loc + bin_mid) <= score_threshold) {
                        bin_min = bin_mid;
                    } else {
                        bin_max = bin_mid;
                    }
                    bin_mid = Math.floor((bin_max - bin_min) / 2 + bin_min);
                }
                // Use the result from this iteration as the maximum for the next.
                bin_max = bin_mid;
                var start = Math.max(1, loc - bin_mid + 1);
                var finish = Math.min(loc + bin_mid, text.length) + pattern.length;

                var rd = Array(finish + 2);
                rd[finish + 1] = (1 << d) - 1;
                for (var j = finish; j >= start; j--) {
                    // The alphabet (s) is a sparse hash, so the following line generates
                    // warnings.
                    var charMatch = s[text.charAt(j - 1)];
                    if (d === 0) {    // First pass: exact match.
                        rd[j] = ((rd[j + 1] << 1) | 1) & charMatch;
                    } else {    // Subsequent passes: fuzzy match.
                        rd[j] = (((rd[j + 1] << 1) | 1) & charMatch) |
                            (((last_rd[j + 1] | last_rd[j]) << 1) | 1) |
                            last_rd[j + 1];
                    }
                    if (rd[j] & matchmask) {
                        var score = match_bitapScore_(d, j - 1);
                        // This match will almost certainly be better than any existing match.
                        // But check anyway.
                        if (score <= score_threshold) {
                            // Told you so.
                            score_threshold = score;
                            best_loc = j - 1;
                            if (best_loc > loc) {
                                // When passing loc, don't exceed our current distance from loc.
                                start = Math.max(1, 2 * loc - best_loc);
                            } else {
                                // Already passed loc, downhill from here on in.
                                break;
                            }
                        }
                    }
                }
                // No hope for a (better) match at greater error levels.
                if (match_bitapScore_(d + 1, loc) > score_threshold) {
                    break;
                }
                last_rd = rd;
            }

            return (best_loc < 0) ? false : true;
        };

    });






    require.alias("component-classes/index.js", "list.fuzzysearch.js/deps/classes/index.js");
    require.alias("component-classes/index.js", "classes/index.js");
    require.alias("component-indexof/index.js", "component-classes/deps/indexof/index.js");

    require.alias("segmentio-extend/index.js", "list.fuzzysearch.js/deps/extend/index.js");
    require.alias("segmentio-extend/index.js", "extend/index.js");

    require.alias("javve-events/index.js", "list.fuzzysearch.js/deps/events/index.js");
    require.alias("javve-events/index.js", "events/index.js");
    require.alias("component-event/index.js", "javve-events/deps/event/index.js");

    require.alias("timoxley-is-collection/index.js", "javve-events/deps/is-collection/index.js");
    require.alias("component-type/index.js", "timoxley-is-collection/deps/type/index.js");

    require.alias("javve-get-by-class/index.js", "list.fuzzysearch.js/deps/get-by-class/index.js");
    require.alias("javve-get-by-class/index.js", "get-by-class/index.js");

    require.alias("javve-to-string/index.js", "list.fuzzysearch.js/deps/to-string/index.js");
    require.alias("javve-to-string/index.js", "list.fuzzysearch.js/deps/to-string/index.js");
    require.alias("javve-to-string/index.js", "to-string/index.js");
    require.alias("javve-to-string/index.js", "javve-to-string/index.js");
    require.alias("list.fuzzysearch.js/index.js", "list.fuzzysearch.js/index.js");if (typeof exports == "object") {
        module.exports = require("list.fuzzysearch.js");
    } else if (typeof define == "function" && define.amd) {
        define(function(){ return require("list.fuzzysearch.js"); });
    } else {
        this["ListFuzzySearch"] = require("list.fuzzysearch.js");
    }})();